"""Colors codes for the flags."""
import re


class ColorsBase:
    """
    Base class for all the color classes for the flags.

    Parameters
    ----------
    colors_codes: Dictionary.
        Hex codes of the colors in the flag. Pairs of name of the color, hex
        code.

    """

    def __init__(self, colors_codes):
        """Define colors and convert to output."""
        # Set color hex codes.
        self.colors_codes = colors_codes

        # Foreground and background colors.
        self.frg = dict()
        self.bkg = dict()

        for color, hexcode in self.colors_codes.items():
            rgb_codes = self.hex2rgb(hexcode)

            self.frg[color] = (
                f"\033[38;2;{rgb_codes[0]};{rgb_codes[1]};{rgb_codes[2]}m"
            )
            self.bkg[color] = (
                f"\033[48;2;{rgb_codes[0]};{rgb_codes[1]};{rgb_codes[2]}m"
            )

        # Font config.
        self.bold = "\033[1m"

        self.frg["BT"] = "\033[38;2;0;0;0m"
        self.frg["WT"] = "\033[38;2;255;255;255m"

        # Reset everything.
        self.reset = "\033[0m"

    def hex2rgb(self, color):
        """
        Convert color codes from Hex to rgb.

        Parameters
        ----------
        color: Str.
            Hex code of the color.

        """
        color = re.match(r"#?([A-Z\d]{6})", color)[1]

        return [int(color[i:i+2], 16) for i in (0, 2, 4)]

    def add_color(self, name, color_code):
        """
        Add a color to the list.

        Parameters
        ----------
        name: Str.
            Name of the color.

        color_code: Str.
            Hex code of the color.

        """
        # Add the code.
        self.colors_codes[name] = color_code

        # Get rgb code.
        rgb_codes = self.hex2rgb(color_code)

        # Add foreground and background.
        self.frg[name] = (
            f"\033[38;2;{rgb_codes[0]};{rgb_codes[1]};{rgb_codes[2]}m"
        )
        self.bkg[name] = (
            f"\033[48;2;{rgb_codes[0]};{rgb_codes[1]};{rgb_codes[2]}m"
        )

    def get_bkg(self, name):
        """Return the background code of the color."""
        return self.bkg[name]

    def get_frg(self, name):
        """Return the foreground code of the color."""
        return self.frg[name]


class RainbowColors (ColorsBase):
    """Colors of the rainbow pride flag (LGBTQ+)."""

    def __init__(self):
        """Define the colors of the rainbow pride flag."""
        super(RainbowColors, self).__init__(
            {
                "RED": "#FF0000",
                "ORANGE": "#FF8E00",
                "YELLOW": "#FFFF00",
                "GREEN": "#008E00",
                "BLUE": "#00C0C0",
                "PURPLE": "#400098"
            }
        )


class ProgressColors (RainbowColors):
    """Colors of the progress pride flag (LGBTQ+)."""

    def __init__(self):
        """Define the colors of the progress pride flag."""
        super(ProgressColors, self).__init__()
        self.add_color("PINK", "#F5A9B8")
        self.add_color("BLUE2", "#5BCEFA")
        self.add_color("WHITE", "#FFFFFF")
        self.add_color("BLACK", "#000000")
        self.add_color("BROWN", "#613915")


class GayColors (ColorsBase):
    """Colors of the gay pride flag (NWLNW)."""

    def __init__(self):
        """Define the colors of the gay pride flag."""
        super(GayColors, self).__init__(
            {
                "DARK_GREEN": "#078D70",
                "GREEN": "#26CEAA",
                "LIGHT_GREEN": "#98E8C1",
                "WHITE": "#FFFFFF",
                "LIGHT_BLUE": "#7BADE2",
                "BLUE": "#5049CC",
                "DARK_BLUE": "#3D1A78"
            }
        )


class LesbianColors (ColorsBase):
    """Colors of the lesbian pride flag (NMLNM)."""

    def __init__(self):
        """Define the colors of the lesbian flag."""
        super(LesbianColors, self).__init__(
            {
                "DARK_ORANGE": "#D52D00",
                "ORANGE": "#EF7627",
                "LIGHT_ORANGE": "#FF9A56",
                "WHITE": "#FFFFFF",
                "LIGHT_PINK": "#D162A4",
                "PINK": "#B55690",
                "DARK_PINK": "#A30262"
            }
        )


class BisexualColors (ColorsBase):
    """Colors of the bisexual pride flag."""

    def __init__(self):
        """Define the colors of the bisexual pride flag."""
        super(BisexualColors, self).__init__(
            {
                "PINK": "#D60270",
                "PURPLE": "#9B4F96",
                "BLUE": "#0038A8"
            }
        )


class TransColors (ColorsBase):
    """Colors of the trans pride flag."""

    def __init__(self):
        """Define the colors of the trans pride flag."""
        super(TransColors, self).__init__(
            {
                "PINK": "#F5A9B8",
                "BLUE": "#5BCEFA",
                "WHITE": "#FFFFFF"
            }
        )


class NonBinaryColors (ColorsBase):
    """Colors of the non-binary pride flag."""

    def __init__(self):
        """Define the colors of the non-binary pride flag."""
        super(NonBinaryColors, self).__init__(
            {
                "YELLOW": "#FCF434",
                "WHITE": "#FFFFFF",
                "PURPLE": "#9C59D1",
                "BLACK": "#2C2C2C"
            }
        )


class GenderFluidColors (ColorsBase):
    """Colors of the gender fluid pride flag."""

    def __init__(self):
        """Define the colors of the gender fluid pride flag."""
        super(GenderFluidColors, self).__init__(
            {
                "PINK": "#FF76A4",
                "WHITE": "#FFFFFF",
                "PURPLE": "#C011D7",
                "BLACK": "#2C2C2C",
                "BLUE": "#2F3CBE"
            }
        )



"""Class of the pride flags."""
from ShellPride import colors


class PrideFlagBase:
    """
    Base class of the pride flag.

    Parameters
    ----------
    flag_colors: ColorsBase.
        Colors of the flag.

    colors_order: Array of string.
        Order of the colors in the flag. It must be names in the colors_codes"
        dictionary.

    length: Int.
        Length of the flag.

    """

    def __init__(self, flag_colors, colors_order, length=25):
        """Define the flag colors and parameters."""
        # Define colors.
        self.flag_colors = flag_colors

        # Define stripes.
        self.stripes = len(colors_order)
        self.colors_order = colors_order

        # Define the length of the flag.
        self.length = length

    def bkg(self, n):
        """Get background code for the n stripe."""
        if isinstance(n, int):
            return self.flag_colors.get_bkg(self.colors_order[n])
        else:
            return self.flag_colors.get_bkg(n)

    def frg(self, n):
        """Get foreground code for the n stripe."""
        if isinstance(n, int):
            return self.flag_colors.get_frg(self.colors_order[n])
        else:
            return self.flag_colors.get_frg(n)

    def center(self, messages, text_colors):
        """
        Center all the messages in the flags colors.

        Parameters
        ----------
        messages: Array of strings.
            Messages to be in the flag, it must be the same length as the
            number of stripes.

        text_colors: Array of strings.
            Colors names of the text.

        """
        max_length = max(
            [len(message) for message in messages] + [self.length]
        )

        return [
            # self.flag_colors.bold +
            self.bkg(i) +
            self.frg(text_colors[i]) +
            messages[i].center(max_length) +
            self.flag_colors.reset
            for i in range(self.stripes)
        ]


class RainbowFlag (PrideFlagBase):
    """
    Class of the rainbow pride flag (LGBTQ+).

    Parameters
    ----------
    length: Int.
        Length of the flag.

    """

    def __init__(self, length=30):
        """Define the flag colors and parameters."""
        super().__init__(
            colors.RainbowColors(),
            [
                "RED",
                "ORANGE",
                "YELLOW",
                "GREEN",
                "BLUE",
                "PURPLE"
            ],
            length=length
        )

    def center(self, messages):
        """
        Center all the messages in the flags colors.

        Parameters
        ----------
        messages: Array of strings.
            Messages to be in the flag, it must be the same length as the
            number of stripes.

        """
        return super().center(messages, ["WT", "BT", "BT", "BT", "BT", "WT"])


class ProgressFlag (PrideFlagBase):
    """
    Class of the progress pride flag (LGBTQ+).

    Parameters
    ----------
    length: Int.
        Length of the flag.

    """

    def __init__(self, length=30):
        """Define the flag colors and parameters."""
        super().__init__(
            colors.ProgressColors(),
            [
                "RED",
                "ORANGE",
                "YELLOW",
                "GREEN",
                "BLUE",
                "PURPLE"
            ],
            length=length
        )

    def center(self, messages):
        """
        Center all the messages in the flags colors.

        Parameters
        ----------
        messages: Array of strings.
            Messages to be in the flag, it must be the same length as the
            number of stripes.

        """
        return super().center(messages, ["WT", "BT", "BT", "BT", "BT", "WT"])


class GayFlagL (PrideFlagBase):
    """
    Class of the gay pride flag (NWLNW). Seven stripes.

    Parameters
    ----------
    length: Int.
        Length of the flag.

    """

    def __init__(self, length=30):
        """Define the flag colors and parameters."""
        super().__init__(
            colors.GayColors(),
            [
                "DARK_GREEN",
                "GREEN",
                "LIGHT_GREEN",
                "WHITE",
                "LIGHT_BLUE",
                "BLUE",
                "DARK_BLUE"
            ],
            length=length
        )

    def center(self, messages):
        """
        Center all the messages in the flags colors.

        Parameters
        ----------
        messages: Array of strings.
            Messages to be in the flag, it must be the same length as the
            number of stripes.

        """
        return super().center(
            messages, ["WT", "BT", "BT", "BT", "BT", "BT", "WT"]
        )


class GayFlagS (PrideFlagBase):
    """
    Class of the gay pride flag (NWLNW). five stripes.

    Parameters
    ----------
    length: Int.
        Length of the flag.

    """

    def __init__(self, length=30):
        """Define the flag colors and parameters."""
        super().__init__(
            colors.GayColors(),
            [
                "DARK_GREEN",
                "LIGHT_GREEN",
                "WHITE",
                "LIGHT_BLUE",
                "DARK_BLUE"
            ],
            length=length
        )

    def center(self, messages):
        """
        Center all the messages in the flags colors.

        Parameters
        ----------
        messages: Array of strings.
            Messages to be in the flag, it must be the same length as the
            number of stripes.

        """
        return super().center(
            messages, ["WT", "BT", "BT", "BT", "WT"]
        )


class LesbianFlagL (PrideFlagBase):
    """
    Class of the lesbian pride flag (NMLNM). Seven stripes.

    Parameters
    ----------
    length: Int.
        Length of the flag.

    """

    def __init__(self, length=30):
        """Define the flag colors and parameters."""
        super().__init__(
            colors.LesbianColors(),
            [
                "DARK_ORANGE",
                "ORANGE",
                "LIGHT_ORANGE",
                "WHITE",
                "LIGHT_PINK",
                "PINK",
                "DARK_PINK"
            ],
            length=length
        )

    def center(self, messages):
        """
        Center all the messages in the flags colors.

        Parameters
        ----------
        messages: Array of strings.
            Messages to be in the flag, it must be the same length as the
            number of stripes.

        """
        return super().center(
            messages, ["WT", "BT", "BT", "BT", "BT", "BT", "WT"]
        )


class LesbianFlagS (PrideFlagBase):
    """
    Class of the lesbian pride flag (NMLNM). five stripes.

    Parameters
    ----------
    length: Int.
        Length of the flag.

    """

    def __init__(self, length=30):
        """Define the flag colors and parameters."""
        super().__init__(
            colors.LesbianColors(),
            [
                "DARK_ORANGE",
                "LIGHT_ORANGE",
                "WHITE",
                "LIGHT_PINK",
                "DARK_PINK"
            ],
            length=length
        )

    def center(self, messages):
        """
        Center all the messages in the flags colors.

        Parameters
        ----------
        messages: Array of strings.
            Messages to be in the flag, it must be the same length as the
            number of stripes.

        """
        return super().center(
            messages, ["WT", "BT", "BT", "BT", "WT"]
        )


class BisexualFlag (PrideFlagBase):
    """
    Class of the bisexual pride flag.

    Parameters
    ----------
    length: Int.
        Length of the flag.

    """

    def __init__(self, length=30):
        """Define the flag colors and parameters."""
        super().__init__(
            colors.BisexualColors(),
            [
                "PINK",
                "PINK",
                "PURPLE",
                "BLUE",
                "BLUE"
            ],
            length=length
        )

    def center(self, messages):
        """
        Center all the messages in the flags colors.

        Parameters
        ----------
        messages: Array of strings.
            Messages to be in the flag, it must be the same length as the
            number of stripes.

        """
        return super().center(
            messages, ["BT", "BT", "BT", "WT", "WT"]
        )


class TransFlag (PrideFlagBase):
    """
    Class of the trans pride flag.

    Parameters
    ----------
    length: Int.
        Length of the flag.

    """

    def __init__(self, length=30):
        """Define the flag colors and parameters."""
        super().__init__(
            colors.TransColors(),
            [
                "BLUE",
                "PINK",
                "WHITE",
                "PINK",
                "BLUE"
            ],
            length=length
        )

    def center(self, messages):
        """
        Center all the messages in the flags colors.

        Parameters
        ----------
        messages: Array of strings.
            Messages to be in the flag, it must be the same length as the
            number of stripes.

        """
        return super().center(
            messages, ["BT", "BT", "BT", "BT", "BT"]
        )


class NonBinaryFlag (PrideFlagBase):
    """
    Class of the non-binary pride flag.

    Parameters
    ----------
    length: Int.
        Length of the flag.

    """

    def __init__(self, length=30):
        """Define the flag colors and parameters."""
        super().__init__(
            colors.NonBinaryColors(),
            [
                "YELLOW",
                "WHITE",
                "PURPLE",
                "BLACK"
            ],
            length=length
        )

    def center(self, messages):
        """
        Center all the messages in the flags colors.

        Parameters
        ----------
        messages: Array of strings.
            Messages to be in the flag, it must be the same length as the
            number of stripes.

        """
        return super().center(
            messages, ["BT", "BT", "BT", "WT"]
        )


class GenderFluidFlag (PrideFlagBase):
    """
    Class of the gender fluid pride flag.

    Parameters
    ----------
    length: Int.
        Length of the flag.

    """

    def __init__(self, length=30):
        """Define the flag colors and parameters."""
        super().__init__(
            colors.GenderFluidColors(),
            [
                "PINK",
                "WHITE",
                "PURPLE",
                "BLACK",
                "BLUE"
            ],
            length=length
        )

    def center(self, messages):
        """
        Center all the messages in the flags colors.

        Parameters
        ----------
        messages: Array of strings.
            Messages to be in the flag, it must be the same length as the
            number of stripes.

        """
        return super().center(
            messages, ["BT", "BT", "WT", "WT", "WT"]
        )


# Set the flag names.
flags_names = {
    "rainbow": RainbowFlag,
    # "progressive": ProgressiveFlag,
    "gayL": GayFlagL,
    "gay": GayFlagS,
    "lesbianL": LesbianFlagL,
    "lesbian": LesbianFlagS,
    "bisexual": BisexualFlag,
    "trans": TransFlag,
    "nonbinary": NonBinaryFlag,
    "genderfluid": GenderFluidFlag,
}



"""Pride Colors."""
import argparse
import sys
import re

from ShellPride.flags import flags_names
from ShellPride.messages import flags_messages
from ShellPride.colors import ColorsBase


def print_flag_list():
    """Print the list of available pride flags."""
    print("Possible flag options:\n\n", end="\t")

    i = 1
    for flag_name in flags_messages.keys():
        print(f"* {flag_name}".ljust(20), end=" ")
        if i % 5 == 0:
            print(end="\n\t")

        i += 1

    print()


def check_flag(name):
    """Check that the flag exists."""
    if name in flags_messages.keys():
        return name

    else:
        print(f"Error: Flag {name} not recognized.\n", file=sys.stderr)
        print_flag_list()
        exit(1)


def add_decorations(args):
    """
    Add decorations for the print.

    Parameters
    ----------
    arg: Shell arguments.
        Arguments passed in the shell.

    """
    # Create a base color.
    base_colors = ColorsBase({"decorations": args.decorator})

    # Create an empty list for the decorations.
    decorations = []

    if args.name:
        decorations.append(
            f"{base_colors.bold + base_colors.get_frg('WT')}Name: "
            f"{base_colors.reset + base_colors.get_frg('decorations')}"
            f"{args.name}"
        )

    if args.pronouns:
        decorations.append(
            f"{base_colors.bold + base_colors.get_frg('WT')}Pronouns: "
            f"{base_colors.reset + base_colors.get_frg('decorations')}"
            f"{args.pronouns}"
        )

    if args.zodiac_solar:
        decorations.append(
            f"{base_colors.bold + base_colors.get_frg('WT')}Zodiac ☀: "
            f"{base_colors.reset + base_colors.get_frg('decorations')}"
            f"{args.zodiac_solar}"
        )

    if args.zodiac_ascendant:
        decorations.append(
            f"{base_colors.bold + base_colors.get_frg('WT')}Zodiac ⬆: "
            f"{base_colors.reset + base_colors.get_frg('decorations')}"
            f"{args.zodiac_ascendant}"
        )

    if args.zodiac_moon:
        decorations.append(
            f"{base_colors.bold + base_colors.get_frg('WT')}Zodiac ☾: "
            f"{base_colors.reset + base_colors.get_frg('decorations')}"
            f"{args.zodiac_moon}"
        )

    return decorations


# Create the parser for the console arguments.
parser = argparse.ArgumentParser(
    prog="shell_pride",
    description="Print shell decorator.",
    formatter_class=argparse.RawTextHelpFormatter
)

# Add argument for the flags that will be printed.
parser.add_argument(
    "-f",  "--flags", type=check_flag, nargs="+", required=True,
    help="List of flags to be shown."
)

# Add argument for the flags that will be printed.
parser.add_argument(
    "-L",  "--length", type=int, default=25,
    help="Length of the flags."
)

# Add argument for the flags that will be printed.
parser.add_argument(
    "-lf",  "--list_flags", action="store_true",
    help="List all the posible flags and exit."
)

# Add argument for the decoration color.
parser.add_argument(
    "-d",  "--decorator", type=str, default="#F5A9B8",
    help="Add the color for the decorator letters."
)

# Add argument for the name.
parser.add_argument("-n",  "--name", type=str, help="Add your name.")

# Add argument for the pronouns.
parser.add_argument("-p",  "--pronouns", type=str, help="Add your pronouns.")

# Add argument for the zodiac sign.
parser.add_argument(
    "-zs",  "--zodiac_solar", type=str, help="Add your solar zodiac sign."
)
parser.add_argument(
    "-za",  "--zodiac_ascendant", type=str,
    help="Add your ascendant zodiac sign."
)
parser.add_argument(
    "-zm",  "--zodiac_moon", type=str, help="Add your moon zodiac sign."
)

# Parse arguments.
args = parser.parse_args()


def main():
    """Print pride flags."""
    # List pride flags.
    if args.list_flags:
        print_flag_list()
        exit(0)

    # Initialize the flags.
    added_flags = [
        flags_names[
            re.search(r"([^\W_\d]+)(_var[\d+])?", name)[1]
        ](length=args.length) for name in args.flags
    ]

    # Get max length.
    max_stripes = max([flag.stripes for flag in added_flags])

    # Get Stripes.
    stripes = [
        flag.center(flags_messages[name])
        for flag, name in zip(added_flags, args.flags)
    ]

    # Add decorations.
    decorations = add_decorations(args)

    # Print the flags.
    print()
    for row in range(max_stripes):
        print(end="\t")

        for col in range(len(stripes)):
            stripe = stripes[col]

            if row < len(stripe):
                print(stripe[row], end="    ")
            else:
                print("".center(args.length), end="    ")

        if row < len(decorations):
            print(decorations[row], end="")

        print()


if __name__ == "__main__":
    main()
